@@ -1,17 +1,17 @@
 from __future__ import absolute_import
 
+import itertools
 import json
 import math
 import typing
 import uuid
-from struct import pack
 
 import attr
 import requests
 from requests import Response
 from requests.packages import urllib3  # noqa
 
-from applitools.common import RunningSession, logger
+from applitools.common import RunningSession, logger, Region
 from applitools.common.errors import EyesError
 from applitools.common.match import MatchResult
 from applitools.common.match_window_data import MatchWindowData
@@ -23,14 +23,17 @@ from applitools.common.utils import (
     gzip_compress,
     json_utils,
     urljoin,
+    iteritems,
 )
-from applitools.common.visual_grid import (
+from applitools.common.ultrafastgrid import (
     RenderingInfo,
     RenderRequest,
     RenderStatusResults,
     RunningRender,
     VGResource,
 )
+from applitools.common.utils.compat import raise_from
+from applitools.core.locators import VisualLocatorsData, LOCATORS_TYPE
 
 if typing.TYPE_CHECKING:
     from typing import Text, List, Any, Optional, Dict, Union
@@ -136,7 +139,8 @@ class _RequestCommunicator(object):
         if use_api_key:
             params["apiKey"] = self.api_key
         params.update(kwargs.get("params", {}))
-        headers = kwargs.get("headers", self.headers).copy()
+        headers = self.headers.copy()
+        headers.update(kwargs.get("headers", {}))
         timeout_sec = kwargs.get("timeout", None)
         if timeout_sec is None:
             timeout_sec = datetime_utils.to_sec(self.timeout_ms)
@@ -206,21 +210,16 @@ class _RequestCommunicator(object):
         return self._long_request_loop(url, delay)
 
 
-def prepare_match_data(match_data):
-    # type: (MatchWindowData) -> bytes
-    match_data_json = json_utils.to_json(match_data)
-    logger.debug("MatchWindowData {}".format(match_data_json))
-    match_data_json_bytes = match_data_json.encode("utf-8")  # type: bytes
-    match_data_size_bytes = pack(">L", len(match_data_json_bytes))  # type: bytes
-    return match_data_size_bytes + match_data_json_bytes
-
-
 class ServerConnector(object):
     """
     Provides an API for communication with the Applitools server.
     """
 
-    DEFAULT_HEADERS = {"Accept": "application/json", "Content-Type": "application/json"}
+    DEFAULT_HEADERS = {
+        "Accept": "application/json",
+        "Content-Type": "application/json",
+        "x-applitools-eyes-client": None,
+    }
 
     API_SESSIONS = "api/sessions"
     API_SESSIONS_RUNNING = API_SESSIONS + "/running/"
@@ -251,7 +250,7 @@ class ServerConnector(object):
         else:
             self._com = _RequestCommunicator(headers=ServerConnector.DEFAULT_HEADERS)
 
-    def update_config(self, conf, render_info=None, ua_string=None):
+    def update_config(self, conf, full_agent_id, render_info=None, ua_string=None):
         if conf.api_key is None:
             raise EyesError(
                 "API key not set! Log in to https://applitools.com to obtain your"
@@ -262,6 +261,7 @@ class ServerConnector(object):
         self._com.timeout_ms = conf._timeout
         self._render_info = render_info
         self._ua_string = ua_string
+        self.DEFAULT_HEADERS["x-applitools-eyes-client"] = full_agent_id
 
     @property
     def server_url(self):
@@ -310,10 +310,12 @@ class ServerConnector(object):
         logger.debug("start_session called.")
         data = json_utils.to_json(session_start_info)
         response = self._com.long_request(
-            "post", url_resource=self.API_SESSIONS_RUNNING, data=data
+            "post",
+            url_resource=self.API_SESSIONS_RUNNING,
+            data=data,
+            headers=self.DEFAULT_HEADERS,
         )
         running_session = json_utils.attr_from_response(response, RunningSession)
-        running_session.is_new_session = response.status_code == requests.codes.created
         self._is_session_started = True
         return running_session
 
@@ -359,7 +361,7 @@ class ServerConnector(object):
                 target_url = rendering_info.results_url
                 guid = uuid.uuid4()
                 target_url = target_url.replace("__random__", str(guid))
-                logger.info("uploading {} to {}".format(media_type, target_url))
+                logger.debug("Uploading {} to {}".format(media_type, target_url))
                 if self._upload_data(
                     bytes_data, rendering_info, target_url, content_type, media_type
                 ):
@@ -368,7 +370,11 @@ class ServerConnector(object):
                 logger.error("Error uploading {}".format(media_type))
                 logger.exception(e)
 
-    @datetime_utils.retry(delays=(0.5, 1, 10), exception=EyesError, report=logger.debug)
+    @datetime_utils.retry(
+        delays=itertools.chain((1000,), (5000,) * 4, (10000,) * 4),
+        exception=EyesError,
+        report=logger.debug,
+    )
     def _upload_data(
         self, data_bytes, rendering_info, target_url, content_type, media_type
     ):
@@ -390,12 +396,19 @@ class ServerConnector(object):
             verify=False,
         )
         if response.status_code in [requests.codes.ok, requests.codes.created]:
-            logger.info("Upload Status Code: {}".format(response.status_code))
+            logger.debug("Upload Status Code: {}".format(response.status_code))
             return True
         raise EyesError(
-            "Failed to Upload Image. Status Code: {}".format(response.status_code)
+            "Failed to Upload. Status Code: {}".format(response.status_code)
         )
 
+    def try_upload_image(self, data):
+        # type: (bytes) -> Optional[Text]
+        try:
+            return self._try_upload_data(data, "image/png", "image/png")
+        except EyesError as e:
+            raise_from(EyesError("Failed to Upload Image"), e)
+
     def match_window(self, running_session, match_data):
         # type: (RunningSession, MatchWindowData) -> MatchResult
         """
@@ -414,18 +427,18 @@ class ServerConnector(object):
             raise EyesError("Session not started")
         app_output = match_data.app_output
         # when screenshot_url is present we don't need to upload again
-        if app_output.screenshot_url is None:
-            app_output.screenshot_url = self._try_upload_data(
-                match_data.app_output.screenshot_bytes, "image/png", "image/png"
+        if app_output.screenshot_url is None and app_output.screenshot_bytes:
+            app_output.screenshot_url = self.try_upload_image(
+                match_data.app_output.screenshot_bytes
             )
+
         if app_output.screenshot_url is None:
             raise EyesError(
                 "MatchWindow failed: could not upload image to storage service."
             )
-        data = prepare_match_data(match_data)
-        # Using the default headers, but modifying the "content type" to binary
+        logger.info("Screenshot image URL: {}".format(app_output.screenshot_url))
+        data = json_utils.to_json(match_data)
         headers = ServerConnector.DEFAULT_HEADERS.copy()
-        headers["Content-Type"] = "application/octet-stream"
         response = self._com.long_request(
             "post",
             url_resource=urljoin(self.API_SESSIONS_RUNNING, running_session.id),
@@ -569,3 +582,13 @@ class ServerConnector(object):
             )
         # TODO: improve parser to handle similar names
         return json_utils.attr_from_response(response, RenderStatusResults)
+
+    def post_locators(self, visual_locators_data):
+        # type: (VisualLocatorsData) -> LOCATORS_TYPE
+        data = json_utils.to_json(visual_locators_data)
+        response = self._com.long_request("post", "api/locators/locate", data=data)
+        response.raise_for_status()
+        return {
+            locator_id: json_utils.attr_from_dict(regions, Region)
+            for locator_id, regions in iteritems(response.json())
+        }
